@@ -84,8 +84,9 @@ zmq::fd_t zmq::open_socket (int domain_, int type_, int protocol_)
 #if defined ZMQ_HAVE_WINDOWS && defined WSA_FLAG_NO_HANDLE_INHERIT
     // if supported, create socket with WSA_FLAG_NO_HANDLE_INHERIT, such that
     // the race condition in making it non-inheritable later is avoided
-    const fd_t s = WSASocket (domain_, type_, protocol_, NULL, 0,
-                              WSA_FLAG_NO_HANDLE_INHERIT);
+    const fd_t s =
+      WSASocket (domain_, type_, protocol_, NULL, 0,
+                 WSA_FLAG_OVERLAPPED || WSA_FLAG_NO_HANDLE_INHERIT);
 #else
     const fd_t s = socket (domain_, type_, protocol_);
 #endif
@@ -128,7 +129,8 @@ void zmq::enable_ipv4_mapping (fd_t s_)
 {
     LIBZMQ_UNUSED (s_);
 
-#if defined IPV6_V6ONLY && !defined ZMQ_HAVE_OPENBSD
+#if defined IPV6_V6ONLY && !defined ZMQ_HAVE_OPENBSD                           \
+  && !defined ZMQ_HAVE_DRAGONFLY
 #ifdef ZMQ_HAVE_WINDOWS
     DWORD flag = 0;
 #else
@@ -232,20 +234,12 @@ int zmq::bind_to_device (fd_t s_, const std::string &bound_device_)
 #ifdef ZMQ_HAVE_SO_BINDTODEVICE
     int rc = setsockopt (s_, SOL_SOCKET, SO_BINDTODEVICE,
                          bound_device_.c_str (), bound_device_.length ());
-
-#ifdef ZMQ_HAVE_WINDOWS
-    if (rc != SOCKET_ERROR)
+    if (rc != 0) {
+        assert_success_or_recoverable (s_, rc);
+        return -1;
+    } else {
         return 0;
-    const int lastError = WSAGetLastError ();
-    errno = wsa_error_to_errno (lastError);
-    wsa_assert (lastError != WSAENOTSOCK);
-    return -1;
-#else
-    if (rc == 0)
-        return 0;
-    errno_assert (errno != ENOTSOCK);
-    return -1;
-#endif
+    }
 #else
     LIBZMQ_UNUSED (s_);
     LIBZMQ_UNUSED (bound_device_);
@@ -679,3 +673,54 @@ void zmq::make_socket_noninheritable (fd_t sock_)
     LIBZMQ_UNUSED (sock_);
 #endif
 }
+
+void zmq::assert_success_or_recoverable (zmq::fd_t s_, int rc_)
+{
+#ifdef ZMQ_HAVE_WINDOWS
+    if (rc_ != SOCKET_ERROR) {
+        return;
+    }
+#else
+    if (rc_ != -1) {
+        return;
+    }
+#endif
+
+    //  Check whether an error occurred
+    int err = 0;
+#if defined ZMQ_HAVE_HPUX || defined ZMQ_HAVE_VXWORKS
+    int len = sizeof err;
+#else
+    socklen_t len = sizeof err;
+#endif
+
+    int rc = getsockopt (s_, SOL_SOCKET, SO_ERROR,
+                         reinterpret_cast<char *> (&err), &len);
+
+    //  Assert if the error was caused by 0MQ bug.
+    //  Networking problems are OK. No need to assert.
+#ifdef ZMQ_HAVE_WINDOWS
+    zmq_assert (rc == 0);
+    if (err != 0) {
+        wsa_assert (err == WSAECONNREFUSED || err == WSAECONNRESET
+                    || err == WSAECONNABORTED || err == WSAEINTR
+                    || err == WSAETIMEDOUT || err == WSAEHOSTUNREACH
+                    || err == WSAENETUNREACH || err == WSAENETDOWN
+                    || err == WSAENETRESET || err == WSAEACCES
+                    || err == WSAEINVAL || err == WSAEADDRINUSE);
+    }
+#else
+    //  Following code should handle both Berkeley-derived socket
+    //  implementations and Solaris.
+    if (rc == -1)
+        err = errno;
+    if (err != 0) {
+        errno = err;
+        errno_assert (errno == ECONNREFUSED || errno == ECONNRESET
+                      || errno == ECONNABORTED || errno == EINTR
+                      || errno == ETIMEDOUT || errno == EHOSTUNREACH
+                      || errno == ENETUNREACH || errno == ENETDOWN
+                      || errno == ENETRESET || errno == EINVAL);
+    }
+#endif
+}
