@@ -49,12 +49,10 @@ const struct sockaddr *zmq::ip_addr_t::as_sockaddr () const
     return &generic;
 }
 
-socklen_t zmq::ip_addr_t::sockaddr_len () const
+zmq::zmq_socklen_t zmq::ip_addr_t::sockaddr_len () const
 {
-    if (family () == AF_INET6) {
-        return sizeof (ipv6);
-    }
-    return sizeof (ipv4);
+    return static_cast<zmq_socklen_t> (family () == AF_INET6 ? sizeof (ipv6)
+                                                             : sizeof (ipv4));
 }
 
 void zmq::ip_addr_t::set_port (uint16_t port_)
@@ -99,7 +97,8 @@ zmq::ip_resolver_options_t::ip_resolver_options_t () :
     _nic_name_allowed (false),
     _ipv6_wanted (false),
     _port_expected (false),
-    _dns_allowed (false)
+    _dns_allowed (false),
+    _path_allowed (false)
 {
 }
 
@@ -143,6 +142,13 @@ zmq::ip_resolver_options_t &zmq::ip_resolver_options_t::allow_dns (bool allow_)
     return *this;
 }
 
+zmq::ip_resolver_options_t &zmq::ip_resolver_options_t::allow_path (bool allow_)
+{
+    _path_allowed = allow_;
+
+    return *this;
+}
+
 bool zmq::ip_resolver_options_t::bindable ()
 {
     return _bindable_wanted;
@@ -168,6 +174,11 @@ bool zmq::ip_resolver_options_t::allow_dns ()
     return _dns_allowed;
 }
 
+bool zmq::ip_resolver_options_t::allow_path ()
+{
+    return _path_allowed;
+}
+
 zmq::ip_resolver_t::ip_resolver_t (ip_resolver_options_t opts_) :
     _options (opts_)
 {
@@ -216,6 +227,13 @@ int zmq::ip_resolver_t::resolve (ip_addr_t *ip_addr_, const char *name_)
         port = 0;
     }
 
+    // Check if path is allowed in ip address, if allowed it must be truncated
+    if (_options.allow_path ()) {
+        size_t pos = addr.find ('/');
+        if (pos != std::string::npos)
+            addr = addr.substr (0, pos);
+    }
+
     //  Trim any square brackets surrounding the address. Used for
     //  IPv6 addresses to remove the confusion with the port
     //  delimiter.
