@@ -243,6 +243,8 @@ zmq::options_t::options_t () :
     zap_enforce_domain (false),
     loopback_fastpath (false),
     multicast_loop (true),
+    in_batch_size (8192),
+    out_batch_size (8192),
     zero_copy (true),
     router_notify (0),
     monitor_event_version (1)
@@ -461,6 +463,24 @@ int zmq::options_t::setsockopt (int option_,
             return do_setsockopt_string_allow_empty_strict (
               optval_, optvallen_, &socks_proxy_address, SIZE_MAX);
 
+        case ZMQ_SOCKS_USERNAME:
+            /* Make empty string or NULL equivalent. */
+            if (optval_ == NULL || optvallen_ == 0) {
+                socks_proxy_username.clear ();
+                return 0;
+            } else {
+                return do_setsockopt_string_allow_empty_strict (
+                  optval_, optvallen_, &socks_proxy_username, 255);
+            }
+        case ZMQ_SOCKS_PASSWORD:
+            /* Make empty string or NULL equivalent. */
+            if (optval_ == NULL || optvallen_ == 0) {
+                socks_proxy_password.clear ();
+                return 0;
+            } else {
+                return do_setsockopt_string_allow_empty_strict (
+                  optval_, optvallen_, &socks_proxy_password, 255);
+            }
         case ZMQ_TCP_KEEPALIVE:
             if (is_int && (value == -1 || value == 0 || value == 1)) {
                 tcp_keepalive = value;
@@ -750,6 +770,22 @@ int zmq::options_t::setsockopt (int option_,
             return do_setsockopt_int_as_bool_relaxed (optval_, optvallen_,
                                                       &multicast_loop);
 
+#ifdef ZMQ_BUILD_DRAFT_API
+        case ZMQ_IN_BATCH_SIZE:
+            if (is_int && value > 0) {
+                in_batch_size = value;
+                return 0;
+            }
+            break;
+
+        case ZMQ_OUT_BATCH_SIZE:
+            if (is_int && value > 0) {
+                out_batch_size = value;
+                return 0;
+            }
+            break;
+#endif
+
         default:
 #if defined(ZMQ_ACT_MILITANT)
             //  There are valid scenarios for probing with unknown socket option
@@ -761,10 +797,10 @@ int zmq::options_t::setsockopt (int option_,
             break;
     }
 
-            // TODO mechanism should either be set explicitly, or determined when
-            // connecting. currently, it depends on the order of setsockopt calls
-            // if there is some inconsistency, which is confusing. in addition,
-            // the assumed or set mechanism should be queryable (as a socket option)
+        // TODO mechanism should either be set explicitly, or determined when
+        // connecting. currently, it depends on the order of setsockopt calls
+        // if there is some inconsistency, which is confusing. in addition,
+        // the assumed or set mechanism should be queryable (as a socket option)
 
 #if defined(ZMQ_ACT_MILITANT)
     //  There is no valid use case for passing an error back to the application
@@ -959,6 +995,14 @@ int zmq::options_t::getsockopt (int option_,
             return do_getsockopt (optval_, optvallen_, socks_proxy_address);
             break;
 
+        case ZMQ_SOCKS_USERNAME:
+            return do_getsockopt (optval_, optvallen_, socks_proxy_username);
+            break;
+
+        case ZMQ_SOCKS_PASSWORD:
+            return do_getsockopt (optval_, optvallen_, socks_proxy_password);
+            break;
+
         case ZMQ_TCP_KEEPALIVE:
             if (is_int) {
                 *value = tcp_keepalive;
@@ -1158,6 +1202,19 @@ int zmq::options_t::getsockopt (int option_,
                 return 0;
             }
             break;
+        case ZMQ_IN_BATCH_SIZE:
+            if (is_int) {
+                *value = in_batch_size;
+                return 0;
+            }
+            break;
+
+        case ZMQ_OUT_BATCH_SIZE:
+            if (is_int) {
+                *value = out_batch_size;
+                return 0;
+            }
+            break;
 #endif
 
 
