@@ -75,6 +75,8 @@ class socket_poller_t
     int add_fd (fd_t fd_, void *user_data_, short events_);
     int modify_fd (fd_t fd_, short events_);
     int remove_fd (fd_t fd_);
+    // Returns the signaler's fd if there is one, otherwise errors.
+    int signaler_fd (fd_t *fd_);
 
     int wait (event_t *event_, int n_events_, long timeout_);
 
